@@ -303,17 +303,13 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
 
                     // sorry Cal. They insisted :(
 
-                    int chk = 0;
-                    char ch = candidate.at(chk);
-                    while (ch >= '0' and ch <= '9' and chk < 4){
-                        chk += 1;
-                        ch = candidate.at(chk);
-                    }
-                    if (ch == '-')
+                    int chk = findSplitPoint(candidate);
+                    if (chk > 0)
                     {
-                        int instnum = string2int(candidate.substr(0, 4));
+                        int instnum = string2int(candidate.substr(0, chk));
                         // remove "NNNN-" and .xiz extension for instrument name
-                        string instname = candidate.substr(5, candidate.size() - xizext.size() - 5);
+                        // modified for numbered instruments with < 4 digits
+                        string instname = candidate.substr(chk + 1, candidate.size() - xizext.size() - chk - 1);
                         addtobank(rootID, banknum, instnum - 1, candidate, instname);
                     }
                     else
@@ -741,6 +737,12 @@ void Bank::addDefaultRootDirs()
         ++ i;
     }
     addRootDir(localPath("/banks"));
+
+    while ( i >= 0)
+    {
+        changeRootID(i, (i * 5) + 5);
+        -- i;
+    }
     rescanforbanks();
 }
 
@@ -978,9 +980,6 @@ void Bank::parseConfigFile(XMLwrapper *xml)
 {
     roots.clear();
     hints.clear();
-    size_t tmp_root = xml->getpar("root_current_ID", 0, 0, 127);
-    size_t tmp_bank = xml->getpar("bank_current_ID", 0, 0, 127);
-
     string nodename = "BANKROOT";
     for (size_t i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
     {
@@ -1015,17 +1014,11 @@ void Bank::parseConfigFile(XMLwrapper *xml)
     }
 
     rescanforbanks();
-
-    setCurrentRootID(tmp_root); // done this way so loading full set
-    setCurrentBankID(tmp_bank); // doesn't change it - need to investigate!
 }
 
 
 void Bank::saveToConfigFile(XMLwrapper *xml)
 {
-    xml->addpar(string("root_current_ID"), currentRootID);
-    xml->addpar(string("bank_current_ID"), currentBankID);
-
     for (size_t i = 0; i < MAX_BANK_ROOT_DIRS; i++)
     {
         if (roots.count(i) > 0 && !roots [i].path.empty())
