@@ -24,7 +24,9 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <sstream>
+#include <iostream>
 #include <string.h>
+#include <mutex>
 
 using namespace std;
 
@@ -235,6 +237,21 @@ string MiscFuncs::findleafname(string name)
 }
 
 
+int MiscFuncs::findSplitPoint(string name)
+{
+    int chk = 0;
+    char ch = name.at(chk);
+    while (ch >= '0' and ch <= '9' and chk < 4)
+    {
+        chk += 1;
+        ch = name.at(chk);
+    }
+    if (ch != '-')
+        chk = 0;
+    return chk;
+}
+
+
 // adds or replaces wrong extension with the right one.
 string MiscFuncs::setExtension(string fname, string ext)
 {
@@ -340,6 +357,81 @@ bool MiscFuncs::matchnMove(int num , char *&pnt, const char *word)
 }
 
 
+/*
+ * These functions provide a transparent text messaging system.
+ * Calling functions only need to recognise integers and strings.
+ *
+ * Pop is destructive. No two functions should ever have been given
+ * the same 'live' ID, but if they do, the second one will get an
+ * empty string.
+ *
+ * Both will block, but should be very quick;
+ *
+ * Normally a message will clear before the next one arrives so the
+ * message numbers should remain very low even over multiple instances.
+ */
+void MiscFuncs::miscMsgInit()
+{
+    for (int i = 0; i < 255; ++i)
+        miscList.push_back("");
+    // we use 255 to denote an invalid entry
+}
+
+int MiscFuncs::miscMsgPush(string _text)
+{
+    mutex mtx;
+    mtx.lock();
+
+    string text = _text;
+    list<string>::iterator it = miscList.begin();
+    int idx = 0;
+
+    while(it != miscList.end())
+    {
+        if ( *it == "")
+        {
+            *it = text;
+            break;
+        }
+        ++ it;
+        ++ idx;
+    }
+    if (it == miscList.end())
+    {
+        cout << "List full :(" << endl;
+        idx = -1;
+    }
+
+    int result = idx; // in case of a new entry before return
+    mtx.unlock();
+    return result;
+}
+
+
+string MiscFuncs::miscMsgPop(int _pos)
+{
+    mutex mtx;
+    mtx.lock();
+
+    int pos = _pos;
+    list<string>::iterator it = miscList.begin();
+    int idx = 0;
+
+    while(it != miscList.end())
+    {
+        if (idx == pos)
+            break;
+        ++ it;
+        ++ idx;
+    }
+    string result = "";
+    if (idx == pos)
+        swap( result, *it); // in case of a new entry before return
+    mtx.unlock();
+    return result;
+}
+
+
 // no more than 32 bit please!
 unsigned int MiscFuncs::nearestPowerOf2(unsigned int x, unsigned int min, unsigned int max)
 {
@@ -357,6 +449,16 @@ unsigned int MiscFuncs::nearestPowerOf2(unsigned int x, unsigned int min, unsign
 }
 
 
+float MiscFuncs::limitsF(float value, float min, float max)
+{
+    if (value > max)
+        value = max;
+    else if (value < min)
+        value = min;
+    return value;
+}
+
+
 unsigned int MiscFuncs::bitFindHigh(unsigned int value)
 {
     unsigned int bit = 0;
