@@ -231,7 +231,13 @@ void JackEngine::registerAudioPort(int partnum)
             //synth->getRuntime().Log("Jack port " + asString(partnum) + " already registered!");
             return;
         }
-        if(synth->part [partnum] && synth->partonoffRead(partnum))
+        /* This has a hack to stop all enabled parts from resistering
+         * individual ports (at startup) if part is not configured for
+         * direct O/P.
+         * Also, at startup there seems to be no way to enable main L/R
+         * without also enabling part 0 direct :(
+         */
+        if(synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1 || partnum == 0))
         {
             string portName = "track_" + asString(partnum + 1) + "_r";
             audio.ports[portnum + 1] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
