@@ -19,7 +19,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code, modified December 2016
 */
 
 #include <iostream>
@@ -50,7 +50,7 @@ ADnoteParameters::ADnoteParameters(FFTwrapper *fft_, SynthEngine *_synth) :
     GlobalPar.AmpEnvelope->ADSRinit_dB(0, 40, 127, 25);
     GlobalPar.AmpLfo = new LFOParams(80, 0, 64, 0, 0, 0, 0, 1, synth);
 
-    GlobalPar.GlobalFilter = new FilterParams(2, 94, 40, synth);
+    GlobalPar.GlobalFilter = new FilterParams(2, 94, 40, 0, synth);
     GlobalPar.FilterEnvelope = new EnvelopeParams(0, 1, synth);
     GlobalPar.FilterEnvelope->ADSRinit_filter(64, 40, 64, 70, 60, 64);
     GlobalPar.FilterLfo = new LFOParams(80, 0, 64, 0, 0, 0, 0, 2, synth);
@@ -189,7 +189,7 @@ void ADnoteParameters::enableVoice(int nvoice)
     VoicePar[nvoice].FreqEnvelope->ASRinit(30, 40, 64, 60);
     VoicePar[nvoice].FreqLfo = new LFOParams(50, 40, 0, 0, 0, 0, 0, 0, synth);
 
-    VoicePar[nvoice].VoiceFilter = new FilterParams(2, 50, 60, synth);
+    VoicePar[nvoice].VoiceFilter = new FilterParams(2, 50, 60, 0, synth);
     VoicePar[nvoice].FilterEnvelope = new EnvelopeParams(0, 0, synth);
     VoicePar[nvoice].FilterEnvelope->ADSRinit_filter(90, 70, 40, 70, 10, 40);
     VoicePar[nvoice].FilterLfo = new LFOParams(50, 20, 64, 0, 0, 0, 0, 2, synth);
@@ -809,3 +809,24 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
         xml->exitbranch();
     }
 }
+
+
+void ADnoteParameters::getLimits(CommandBlock *getData)
+{
+    int control = getData->data.control;
+    //int engine = getData->data.engine;
+    switch (control)
+    {
+        case 32:
+        case 96: // only for advoice modulator
+            getData->limits.min = -8192;
+            getData->limits.max = 8191;
+            break;
+        case 82:
+        case 112:
+        case 136:
+            getData->limits.min = -64;
+            getData->limits.max = 63;
+            break;
+    }
+}
