@@ -25,9 +25,11 @@
 using namespace std;
 
 #include "Misc/Config.h"
+#include "Misc/Splash.h"
 #include "Misc/SynthEngine.h"
 #include "MusicIO/MusicClient.h"
 #include "MasterUI.h"
+#include "UI/MiscGui.h"
 #include "Synth/BodyDisposal.h"
 #include <map>
 #include <list>
@@ -40,11 +42,10 @@ using namespace std;
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Shared_Image.H>
 #include <FL/Fl_PNG_Image.H>
-#include "yoshimi-logo.h"
 
 #include <readline/readline.h>
 #include <readline/history.h>
-#include <Misc/CmdInterface.h>
+#include <Interface/CmdInterface.h>
 
 //extern void cmdIfaceCommandLoop();
 
@@ -108,7 +109,7 @@ static void *mainGuiThread(void *arg)
         FILE *f = fopen(fname, "wb");
         if (f)
         {
-            fwrite(yoshimi_logo_png, sizeof(yoshimi_logo_png), 1, f);
+            fwrite(splashPngData, splashPngLength, 1, f);
             fclose(f);
         }
     }
@@ -116,11 +117,9 @@ static void *mainGuiThread(void *arg)
     if (fname)
     unlink(fname);
 #else
-    Fl_PNG_Image pix("yoshimi_logo_png", yoshimi_logo_png, sizeof(yoshimi_logo_png));
+    Fl_PNG_Image pix("splash_screen_png", splashPngData, splashPngLength);
 #endif
 
-    const int splashWidth = 411;
-    const int splashHeight = 311;
     const int textHeight = 20;
     const int textBorder = 15;
 
@@ -134,6 +133,8 @@ static void *mainGuiThread(void *arg)
     boxLb.align(FL_ALIGN_CENTER);
     boxLb.labelsize(textHeight);
     boxLb.labeltype(FL_NORMAL_LABEL);
+    //boxLb.labelcolor(FL_WHITE);
+    //boxLb.labelfont(FL_HELVETICA | FL_BOLD);
     boxLb.labelfont(FL_HELVETICA | FL_ITALIC);
     string startup = YOSHIMI_VERSION;
     startup = "Yoshimi " + startup + " is starting";
@@ -180,8 +181,32 @@ static void *mainGuiThread(void *arg)
             SynthEngine *_synth = it->first;
             MusicClient *_client = it->second;
             _synth->getRuntime().deadObjects->disposeBodies();
+
+            /*
+             * Setting pad parameters can take very many seconds.
+             * This dodge is to force it to diable the part then
+             * make the change take place in a low priority thread.
+             * It needs to be improved!
+             */
+            unsigned int padApply = _synth->getRuntime().padApply;
+            if (padApply < 0xffff)
+            {
+                _synth->interchange.setpadparams(padApply);
+                _synth->getRuntime().padApply = 0xffff;
+            }
+            // end of pad dodge
+
             if (!_synth->getRuntime().runSynth && _synth->getUniqueId() > 0)
             {
+                if (_synth->getRuntime().configChanged)
+                {
+                    size_t tmpRoot = _synth->ReadBankRoot();
+                    size_t tmpBank = _synth->ReadBank();
+                    _synth->getRuntime().loadConfig(); // restore old settings
+                    _synth->SetBankRoot(tmpRoot);
+                    _synth->SetBank(tmpBank); // but keep current root and bank
+                }
+                _synth->getRuntime().saveConfig();
                 int tmpID =  _synth->getUniqueId();
                 if (_client)
                 {
@@ -229,6 +254,15 @@ static void *mainGuiThread(void *arg)
         else
             usleep(33333);
     }
+    if (firstSynth->getRuntime().configChanged)
+    {
+        size_t tmpRoot = firstSynth->ReadBankRoot();
+        size_t tmpBank = firstSynth->ReadBank();
+        firstSynth->getRuntime().loadConfig(); // restore old settings
+        firstSynth->SetBankRoot(tmpRoot);
+        firstSynth->SetBank(tmpBank); // but keep current root and bank
+    }
+    firstSynth->getRuntime().saveConfig();
     firstSynth->saveHistory();
     firstSynth->saveBanks(0);
     return NULL;
@@ -281,6 +315,10 @@ bool mainCreateNewInstance(unsigned int forceId)
         {
             GuiThreadMsg::sendMessage(synth, GuiThreadMsg::NewSynthEngine, 0);
         }
+        if (synth->getRuntime().audioEngine < 1)
+            fl_alert("Yoshimi can't find an available sound system. Running with no Audio");
+        if (synth->getRuntime().midiEngine < 1)
+            fl_alert("Yoshimi can't find an input system. Running with no MIDI");
     }
 
     synth->getRuntime().StartupReport(musicClient);
